<?php

/**
 * The file that includes all DivideBuy APIs for order and retailer configuration.
 * 
 * @since    1.0
 * @package  Divide_Buy
 * @author   Cygnet Infotech Pvt. Ltd.
 */

class DivideBuy_Api{
        
    /** 
         * Hook WordPress
     * @return void
     */
    public function __construct(){
            add_filter('query_vars', array($this, 'add_query_vars'), 0);
            add_action('parse_request', array($this, 'sniff_requests'), 0);
            add_action('init', array($this, 'add_endpoint'), 0);
    }   
    
    /** 
         * Add public query vars
     * @param array $vars List of current public query vars
     * @return array $vars 
     */
    public function add_query_vars($vars){
            $vars[] = '__dividebuy';
            $vars[] = 'api';
            $vars[] = 'payment';
            $vars[] = 'getorderdetails';
            $vars[] = 'response';
            $vars[] = 'createguarantororder';
            $vars[] = 'verifypostcode';
            $vars[] = '__retailer';
            $vars[] = 'index';
            $vars[] = 'updateMinOrderAmount';
            $vars[] = 'dividebuypage';
            return $vars;
    }
    
    /** 
         *  Add API Endpoint
     *  @return void
     */
    public function add_endpoint(){
            add_rewrite_rule('^dividebuy/api/getorderdetails?','index.php?__dividebuy&api&getorderdetails','top');
            add_rewrite_rule('^dividebuy/api/response?','index.php?__dividebuy&api&response','top');
            add_rewrite_rule('^dividebuy/payment/response?','index.php?__dividebuy&payment&response','top');
            add_rewrite_rule('^dividebuy/api/createguarantororder?','index.php?__dividebuy&api&createguarantororder','top');
            add_rewrite_rule('^dividebuy/api/verifypostcode?','index.php?__dividebuy&api&verifypostcode','top');
            add_rewrite_rule('^dividebuy/api/dividebuypage?','index.php?__dividebuy&api&dividebuypage','top');
            flush_rewrite_rules();
    }
        
    /** 
         * Sniff Requests
     * This is where we hijack all API requests
     * @return die if API request
     */
    public function sniff_requests(){
            global $wp;
            // check if query_var is updateMinOrderAmount
            if(isset($wp->query_vars['index']) && isset($wp->query_vars['updateMinOrderAmount'])){
                $this->update_min_order_amount();
                exit;
            }
            
            // check if query_var is getorderdetails
            if(isset($wp->query_vars['getorderdetails'])){
                $post = trim(file_get_contents("php://input"));       
                $postData = json_decode($post);
                $token = $postData->storeToken;
                $authentication = $postData->storeAuthentication;
                // Authenticate store
                $authentication_status = $this->authenticate_store($token, $authentication);
                if($authentication_status == "true"){
                    $order_id = $postData->orderId;
                    $this->get_order_details($order_id, $token, $authentication);
                }else{
                    $result = array(
                        'error'     => 1,
                        'success'   => 0,
                        'message'   => 'Store authentication failed.',
                        'status'    => '401'
                    );
                    $this->send_response($result);
                }
                exit();
            }
            
            // Managing API calls for order success or cancel.
            if(isset($wp->query_vars['api']) && isset($wp->query_vars['response'])){
                $post = trim(file_get_contents("php://input"));
                $postData = json_decode($post);
                $token = $postData->store_token;
                $authentication = $postData->store_authentication;
                
                // Authenticate store
                $authentication_status = $this->authenticate_store($token, $authentication);
                $order_method = $postData->method;
                
                if($authentication_status == "true"){
                    $order_id = $postData->store_order_id;
                    $email = $postData->customer_email;
                    $address = $postData->address;
                    $order = new WC_Order( $order_id );
                    
                    if($order_method == "orderSuccess"){
                        $order_time = $postData->orderTime;
                        $this->success_order_api($order, $order_time, $email, $address);
                    }elseif($order_method == "orderCancel") {
                        $this->cancel_order_api($order);
                    }elseif($order_method == "orderDelete"){
                        $flagToDeleteOrders = get_option("wc_settings_tab_dividebuy_flag_to_delete_orders");
                        $order->update_status('cancelled', 'Order has been canceled from DivideBuy.');
                        if($flagToDeleteOrders) {
                            //Test Order Deleted
                            $return_data = wp_delete_post($order_id,true);
                        }
                        //After sucessful execution return request
                        $result = array(
                        'error'     => 0,
                        'success'   => 1,
                        'message'   => 'Deleted test order successfully.',
                        );
                        $this->send_response($result);    
                        
                    }elseif($order_method == "retailerConfigurations") {
                        $instalments = array();
                        $i=0;
                        $retailer_configuration_details = $postData->retailerConfigurationDetails;
                        foreach ($retailer_configuration_details as $configuration){
                            if($configuration->type == "global" && $configuration->key == "excludePostCodes"){
                                $this->update_postcodes($configuration->value);
                            }
                            if($configuration->type == "global" && $configuration->key == "taxClass"){
                                $this->update_taxclass($configuration->value);
                            }
                            if($configuration->type == "global" && $configuration->key == "inStoreCollection"){
                                $this->update_shipping_auto($configuration->value);
                            }
                            if($configuration->type == "global" && $configuration->key == "flagToDeleteOrders"){
                                $this->update_flag_to_delete_orders($configuration->value);
                            }
                            if($configuration->type == "themes" && $configuration->key == "logoUrl"){
                                $this->update_log_url($configuration->value);
                            }
                            if($configuration->type == "instalments"){
                                $instalments[$i]['key'] = $configuration->key;
                                $instalments[$i]['value'] = $configuration->value;
                                $i++;
                            }
                        }
                        if(count($instalments) != 0){
                            $this->update_instalments($instalments);
                        }
                        //After Execution return request
                        $result = array(
                        'error'     => 0,
                        'success'   => 1,
                        'message'   => 'Updated successfully.',
                        );
                        $this->send_response($result);

                    }elseif($order_method == "updateCouriers"){
                        $this->update_couriers($postData->couriers);
                    }
                }else{
                    $result = array(
                        'error'     => 1,
                        'success'   => 0,
                        'message'   => 'Store authentication failed.',
                        'status'    => '401'
                    );
                    $this->send_response($result);
                }
                exit();
            }
            
            // Managing redirection when order is successfully placed or canceled.
            if(isset($wp->query_vars['payment']) && isset($wp->query_vars['response'])){
                
                // get value of splash key
                $get_data = base64_decode(filter_input(INPUT_GET, 'splashKey')); 
                $order_data = explode(":", $get_data);
                $order_id = $order_data[0];
                $order = new WC_Order( $order_id );
                $order_status = $order_data[1];
                //getting required details for curl call
                $storeId = get_option('wc_settings_tab_retailer_id');
                $tokenNumber =  get_option('wc_settings_tab_retailer_token_number');
                $authenticationKey = get_option('wc_settings_tab_retailer_authentication_number');
                if($order_status == "success"){
                    //Loading the order
                    $order_data = new WC_Order( $order_id);
                    if($order_data->status == 'freeze-dividebuy')
                    {
                        //api params
                        $requestData = array(
                            "storeOrderId"          => $order_id,
                            "storeAuthentication"   => $authenticationKey,
                            "storeToken"            => $tokenNumber,
                            "retailerStoreCode"     => "",
                        );
                        $data_json = json_encode($requestData);

                        $url = Divide_Buy_Configuration::get_api_url().'api/getuserorder';
                        $method = 'POST';
                        $responseAlldata = $this->curl_call_api($url,$data_json,$method);
                        $responseAlldata = json_decode($responseAlldata,true);
                        $response = json_decode($responseAlldata['data'],true);
                        if($response["method"] == "orderSuccess" && $responseAlldata["status"] == 'ok'){
                            
                            $orderTime = $response['orderTime'];
                            $address = $response['address'];
                            $street1 = $address['house_number'] . ' '. $address['house_name'] . ', ' . $address['street'];
                            $street2 = $address['address2'];

                            //updating the post time
                            wp_update_post(
                                array (
                                    'ID'            => $order_id, // ID of the post to update
                                    'post_date'     => $orderTime,
                                    'post_date_gmt' => get_gmt_from_date( $orderTime )
                                )
                            );
                            //Customer email
                            $customerEmail = $response['customer_email'];
                            $userAddress = array(
                                'first_name' => $address['first_name'],
                                'last_name'  => $address['last_name'],
                                'postcode'   => $address['postcode'],
                                'state'      => $address['region'],
                                'city'       => $address['city'],
                                'email'      => $customerEmail,
                                'phone'      => $address['contact_number'],
                                'address_1'  => trim($street1, ', '),
                                'address_2'  => $street2,
                                'country'    => 'GB'
                            );
                            //Updating the order address
                            $order->set_address( $userAddress, 'billing' );
                            $order->set_address( $userAddress, 'shipping' );
                            //Order status update
                            $order->update_status('processing');
                            $mailer = WC()->mailer();
                            $mails = $mailer->get_emails();
                            if ( ! empty( $mails ) ) {
                                foreach ( $mails as $mail ) {
                                    if ( $mail->id == 'new_order' || $mail->id == 'customer_processing_order' ) {
                                       $mail->trigger( $order->id );
                                    }
                                 }
                            }                            
                        }
                        $this->success_order($order);
                    }else{

                        $this->success_order($order);
                    }
                    //Retailer Sync flag
                    $request = array(                    
                        "storeOrderId"          => $order_id,
                        "storeAuthentication"   => $authenticationKey,
                        "storeToken"            => $tokenNumber
                    );
                    $data_json = json_encode($request);
                    $url = Divide_Buy_Configuration::get_api_url().'api/syncretorder';
                    $method = 'POST';
                    $this->curl_call_api($url,$data_json,$method);
                }elseif ($order_status == "cancel") {
                    $this->cancel_order($order);
                }
                exit();
            }
            
            // Creating guarantor order
            if(isset($wp->query_vars['createguarantororder'])){
                $this->create_guarantor_order();
                exit();
            }

            // Getting shipping methods based on 
            if(isset($wp->query_vars['verifypostcode'])){
                $this->verify_postcode();
                exit();
            }
            //Handling api of create page or update page content
            if(isset($wp->query_vars['dividebuypage']))
            {
                $this->dividebuypage();
            }
    }
    
        /**
         * 
         * @param type $token
         * @param type $authentication
         * @return string bool
         * 
         * This function checks that store is authenticated or not.
         */
        protected function authenticate_store($token, $authentication){
            $is_authenticated = "";
            $store_token = get_option('wc_settings_tab_retailer_token_number');
            $store_authentication = get_option('wc_settings_tab_retailer_authentication_number');
            
            if($token == $store_token && $authentication == $store_authentication){
                $is_authenticated = "true";
            }else{
                $is_authenticated = "false";
            }
            return $is_authenticated;
        }
        
        /**
         * Function for updating minimum order amount field.
         */
        public function update_min_order_amount(){
            $token = $_POST['token_number'];
            $authentication = $_POST['auth_number'];
            $authentication_status = $this->authenticate_store($token, $authentication);
            if($authentication_status == "true"){
                $minimum_order_amount = $_POST['minAmount'];                
                update_option('wc_settings_tab_dividebuy_minimum_order_value', $minimum_order_amount);
            }
        }
        
    /**
         * 
         * @param type $order_id
         * @param type $token
         * @param type $authentication
         * 
         * This function return order details in JSON format based on provided order id.
         */
    protected function get_order_details($order_id, $token, $authentication){
            $order = new WC_Order( $order_id );
            $shipping_data = $order->get_items('shipping');
            foreach($shipping_data as $shipping){
                $shipping_label = $shipping['name'];
                $shipping_method = $shipping['method_id'];
                $shipping_incl_vat = $shipping['cost'] + $shipping['total_tax'];
            }

            $billing_address_array = array($order->billing_first_name, $order->billing_last_name, $order->billing_email, $order->billing_address_1, $order->billing_address_2, $order->billing_postcode, $order->billing_city, $order->billing_state, $order->billing_phone, $order->billing_country);
            $shipping_address_array = array($order->shipping_first_name, $order->shipping_last_name, $order->shipping_email, $order->shipping_address_1, $order->shipping_address_2, $order->shipping_postcode, $order->shipping_city, $order->shipping_state, $order->shipping_phone, $order->shipping_country);

            $address_difference = array_diff($billing_address_array, $shipping_address_array);
            // billing and shipping addresses are different
            if( count($address_difference) == 0 ) {
                $different_address = 0;
            }else{
                $different_address = 1;
            }

            // Getting general information of an order.
            $result = array();
            $result['order_detail']['store_order_id'] = $order_id;
            $result['order_detail']['store_order_increment_id'] = $order_id;
            $result['order_detail']['store_token'] = $token;
            $result['order_detail']['store_authentication'] = $authentication;
            //checking website is havig ssl or not
            $secureFlag = parse_url(get_site_url(), PHP_URL_SCHEME);
            if($secureFlag != "https"){
                $retailerLogoUrl = get_option('wc_settings_tab_dividebuy_logo_url_retailer_api');
            }else{
                $retailerLogoUrl = get_option('wc_settings_tab_retailer_image');    
            }
            $result['order_detail']['logo_url'] = $retailerLogoUrl;
            $result['order_detail']['grand_total'] = (float)$order->get_total();
            $result['order_detail']['subtotal'] = (float)$order->get_subtotal();
            $result['order_detail']['subtotalInclVat'] = number_format( (float) $order->get_total() - (float)$shipping_incl_vat, wc_get_price_decimals(), '.', '' );
            $cart_discount = get_post_meta($order_id, '_cart_discount',true);
            $cart_discount_tax = get_post_meta($order_id, '_cart_discount_tax',true);
            $total_discount = $cart_discount_tax + $cart_discount;
            $result['order_detail']['discount'] = (float)$total_discount;
            $result['order_detail']['discountApplied'] = 'afterVat';
            $result['order_detail']['shipping_label'] = $shipping_label;
            $result['order_detail']['shipping_method'] = $shipping_method;
            $result['order_detail']['shipping'] = (float)$order->get_total_shipping();
            $result['order_detail']['shippingInclVat'] = (float)$shipping_incl_vat;
            $result['order_detail']['is_default_shipping'] = $different_address;
            $result['order_detail']['is_default_billing'] = $different_address;
            $result['order_detail']['vat'] = (float)$order->get_total_tax();
            
            $product_details = $this->get_order_product_details($order);            
            $result['product_details'] = $product_details;
            
            // getting order shipping address.
            $result['shipping_address']['first_name'] = $order->shipping_first_name;
            $result['shipping_address']['last_name'] = $order->shipping_last_name;
            $result['shipping_address']['email'] = $order->shipping_email;
            $result['shipping_address']['street'] = $order->shipping_address_1;
            $result['shipping_address']['postcode'] = $order->shipping_postcode;
            $result['shipping_address']['region'] = $order->shipping_city;
            $result['shipping_address']['city'] = $order->shipping_city;
            
            // getting order billing address.
            $result['billing_address']['first_name'] = $order->billing_first_name;
            $result['billing_address']['last_name'] = $order->billing_last_name;
            $result['billing_address']['email'] = $order->billing_email;
            $result['billing_address']['street'] = $order->billing_address_1;
            $result['billing_address']['postcode'] = $order->billing_postcode;
            $result['billing_address']['region'] = $order->billing_city;
            $result['billing_address']['city'] = $order->billing_city;
            
            $this->send_response($result);
    }
        
        /**
         * 
         * @param type $order
         * @return product details for order.
         */
        protected function get_order_product_details($order) {
            $order_item = $order->get_items();
            $product_details = array();
            $product_count = 0;
            $tax_class = get_option('wc_settings_tab_dividebuy_tax_class');
            // Getting product information of an order.

            foreach ($order_item as $product){
                $product_id = $product['product_id'];
                $product_variation_id = $product['variation_id'];
                
                // Check if product has variation.
                if ($product_variation_id) { 
                    $single_product_data = new WC_Product_Variation($product_variation_id);
                    $product_image = wp_get_attachment_image_src( get_post_thumbnail_id( $product_variation_id ), 'single-post-thumbnail' );
                    $variation = wc_get_product($product_variation_id);
                    $attributes = $variation->get_variation_attributes();
                    if(!empty($attributes)){
                        foreach($attributes as $key => $value){
                            $attribute_key = urldecode(str_replace('attribute_',"", $key));
                            $update_attribute[$attribute_key] = $value;
                        }
                    }
                    $product_weight = get_post_meta($product_variation_id, '_weight', true);
                } else {
                    $single_product_data = new WC_Product($product_id);
                    $product_image = wp_get_attachment_image_src( get_post_thumbnail_id( $product_id ), 'single-post-thumbnail' );
                    $product_weight = get_post_meta($product_id, '_weight', true);
                }

                $product_details[$product_count]['name'] = $product['name'];                
                $product_details[$product_count]['sku'] = $single_product_data->get_sku();                
                $product_details[$product_count]['qty'] = $product['qty'];                
                $product_details[$product_count]['price'] = (float)$single_product_data->get_price_excluding_tax();                
                $product_details[$product_count]['priceInclVat'] = (float)(((float)$product['line_total'] + $product['line_subtotal_tax']) / $product['qty']);                
                $product_details[$product_count]['rowTotal'] = (float)$product['line_total'];                
                $product_details[$product_count]['rowTotalInclVat'] = (float)$product['line_total'] + $product['line_subtotal_tax'];

                // Getting product base discount including vat
                $product_line_subtotal_incl_tax = $product['line_subtotal'] + $product['line_subtotal_tax'];
                $product_line_subtotal_excl_tax = $product['line_total'] + $product['line_tax'];
                $product_discount = $product_line_subtotal_incl_tax - $product_line_subtotal_excl_tax;
                $product_details[$product_count]['discount'] = (float)$product_discount;

                //Product base dividebuy tax
                $dividebuy_tax_class = get_post_meta($product_id, 'product_dividebuy_taxclass',true);
                //Product base default woocommerce tax class
                $product_tax_cname = get_post_meta($product_id, '_tax_class',true);
                if($product_tax_cname!='')
                {
                    //$rates=WC_Tax::get_rates( $product->get_tax_class($product_tax_cname) );
                    //foreach ($rates as $rates) {
                        $rate_percentage = 20;
                    //}
                }
                else
                {
                    //$rates=WC_Tax::get_rates( $product->get_tax_class('standard') );
                    
                    //foreach ($rates as $rates) {
                        $rate_percentage = 20;
                    //}
                }

                $default_tax_class = (float)number_format($rate_percentage, 2, '.', '');
                if($dividebuy_tax_class !='' && isset($dividebuy_tax_class)){
                    $product_details[$product_count]['DivVat'] = $dividebuy_tax_class;
                }elseif($tax_class !=''){
                    $product_details[$product_count]['DivVat'] = $tax_class;
                    
                }else{
                    $product_details[$product_count]['DivVat'] = $default_tax_class;
                }

                $product_details[$product_count]['short_description'] = get_post($product_id)->post_excerpt;
                $product_details[$product_count]['product_type'] = $single_product_data->get_type();
                $product_details[$product_count]['product_weight'] = $product_weight;

                $product_visibility = get_post_meta($product_id, '_visibility',true);
                if($product_visibility == "visible"){
                    $visibility = 4;
                }elseif($product_visibility == "catalog"){
                    $visibility = 2;
                }elseif($product_visibility == "search"){
                    $visibility = 3;
                }elseif($product_visibility == "hidden"){
                    $visibility = 1;
                }

                $product_details[$product_count]['product_visibility'] = $visibility;
                $product_details[$product_count]['image_url'] = $product_image[0];
                
                $product_details[$product_count]['product_options'] = $update_attribute;
                $product_count++;
            }
            return $product_details;
        }
        
        /**
         * 
         * @param type $order
         * Redirect to order success page.
         */
        protected function success_order($order){
            WC()->session->__unset('default_shipping_checkout');
            // Get an instance of the WC_Order object
            $order_data = new WC_Order( $order->ID );           
            if($order_data->status == 'freeze-dividebuy')
            {
                $order->update_status('processing');
                $mailer = WC()->mailer();
                $mails = $mailer->get_emails();
                if ( ! empty( $mails ) ) {
                    foreach ( $mails as $mail ) {
                        if ( $mail->id == 'new_order' || $mail->id == 'customer_processing_order' ) {
                           $mail->trigger( $order->id );
                        }
                     }
                }
            }
            wp_redirect(WC_Payment_Gateway::get_return_url( $order ));
        }
        
        /**
         * 
         * @global type $woocommerce
         * @param type $order
         * cancel order and redirect to cart page.
         */
        protected function cancel_order($order){
            WC()->session->__unset('default_shipping_checkout');
            $cart_url = WC()->cart->get_cart_url();
            $order_item = $order->get_items();
            
            /*foreach ($order_item as $product){
                $product_id = $product['product_id'];
                $product_variation_id = $product['variation_id'];
                
                // Check if product has variation.
                if($product_variation_id){
                    $variation = wc_get_product($product_variation_id);
                    $attributes = $variation->get_variation_attributes();
                    WC()->cart->add_to_cart($product_id, $product['qty'], $product_variation_id, $attributes);
                }else{
                    WC()->cart->add_to_cart($product_id, $product['qty']);
                }
            }*/
            WC()->session->__unset( 'nondividebuy_products_cart_session' );
            wc_add_notice( __( 'Your DivideBuy order has been cancelled for you to amend your basket.', 'woocommerce' ), "error" );
            //if order is cancelled and order from dividebuy than it will delete the order.
            wp_delete_post($order->id,true);
            wp_redirect($cart_url);
        }
        
        /**
         * 
         * @param type $order
         * @param type $email
         * @param type $address
         * Updates order address and customer email address for provided order.
         */
        protected function success_order_api($order, $order_time, $email, $address){            
            wp_update_post(
                array (
                    'ID'            => $order->ID, // ID of the post to update
                    'post_date'     => $order_time,
                    'post_date_gmt' => get_gmt_from_date( $order_time )
                )
            );
            $address_1 = $address->house_number . ' '. $address->house_name . ', ' . $address->street;
            $address_2 = $address->address2;
            $user_address = array(
                'first_name' => $address->first_name,
                'last_name'  => $address->last_name,
                'email'      => $email,
                'phone'      => $address->contact_number,
                'address_1'  => trim($address_1, ', '),
                'address_2'  => $address_2,
                'city'       => $address->city,
                'state'      => $address->region,
                'postcode'   => $address->postcode,
                'country'    => 'GB'
            );
            
            $order->set_address( $user_address, 'billing' );
            $order->set_address( $user_address, 'shipping' );


            // Get an instance of the WC_Order object
            $orderload = wc_get_order( $order->ID );
            $order_data = $orderload->get_data(); // The Order data
            if($order_data['status'] == 'freeze-dividebuy')
            {
                $order->update_status('processing');
                $mailer = WC()->mailer();
                $mails = $mailer->get_emails();
                if ( ! empty( $mails ) ) {
                    foreach ( $mails as $mail ) {
                        if ( $mail->id == 'new_order' || $mail->id == 'customer_processing_order' ) {
                           $mail->trigger( $order->id );
                        }
                     }
                }
            }
            
            $result = array(
                'error'     => 0,
                'success'   => 1,
                'status'    => '200',
                'order_id'  => $order->id,
                'message'   => 'Order placed successfully'
            );
            $this->send_response($result);
        }
        
        /**
         * 
         * @param type $order
         */
        protected function cancel_order_api($order){
            $order_is_dividebuy = get_post_meta($order->id, 'order_is_dividebuy');
            if($order_is_dividebuy[0] != "yes"){
                $result = array(
                    'error'     => 1,
                    'success'   => 0,
                    'order_id'  => $order->id,
                    'message'   => 'Non-DivideBuy order.',
                    'status'    => '402',
                );
            }
            elseif($order->status == "cancelled"){
                $result = array(
                    'error'     => 1,
                    'success'   => 0,
                    'status'    => '405',
                    'order_id'  => $order->id,
                    'message'   => 'order is already cancelled.'
                );
            }else{
                $order->update_status('cancelled', 'Order has been canceled from DivideBuy.');
                $order->update_status('freeze-dividebuy', 'Order has been freezed by DivideBuy after cancel.');
                $result = array(
                    'error'     => 0,
                    'success'   => 1,
                    'status'    => 'ok',
                    'order_id'  => $order->id
                );
                $time_diff      = abs(time() - get_post_time('U', true, $order->id));
                $diff_minutes   = round($time_diff / 60);
                if($diff_minutes <= 1440){
                    wp_delete_post($order->id,true);
                }

            }

            $this->send_response($result);

        }
        /*
        *
        * @param type $logourl
        * Updates logourl field value as per retrieved data
        */
        public function update_log_url($logourl){
            
            update_option('wc_settings_tab_dividebuy_logo_url_retailer_api', $logourl);
        }
        /**
         * 
         * @param type $postcodes
         * Updates postcode field value as per retrieved data
         */
        public function update_postcodes($postcodes){
            
            update_option('wc_settings_tab_dividebuy_checkout_postcode_list', $postcodes);
        }
        
        /**
         * 
         * @param type $tax_class
         * This function updates value of tax class option based on sent request.
         */
        public function update_taxclass($tax_class){
            update_option('wc_settings_tab_dividebuy_tax_class', $tax_class);
        }

        /**
         * 
         * @param type $tax_class
         * This function updates value of tax class option based on sent request.
         */
        public function update_shipping_auto($shipping_auto){
            update_option('wc_settings_tab_auto_shipping', $shipping_auto);
        }

        /**
         * 
         * @param type $flagToDeleteOrders
         * This function updates value of flag to delete orders option based on sent request.
         */
        public function update_flag_to_delete_orders($flagToDeleteOrders){
            update_option('wc_settings_tab_dividebuy_flag_to_delete_orders', $flagToDeleteOrders);
        }

        /**
         * 
         * @param type $instalments
         * This function updates values of retailer's available instalments based on sent request.
         */
        public function update_instalments($instalments){
            $allValues = array();
            for ($k = 0; $k < sizeof($instalments); $k++) {
                $allValues[] = $instalments[$k]['value'];
            }
            if (!empty(min($allValues))) {
                $order_value = min($allValues);
                update_option('wc_settings_tab_dividebuy_minimum_order_value', $order_value);
            }

            
            update_option('wc_settings_tab_dividebuy_retailer_installments', serialize($instalments));
        }
        
        public function update_couriers($couriers){
            $courier_list = "";
            foreach($couriers as $courier){
                $courier_list = $courier_list.$courier.',';
            }
            $courier_list = rtrim($courier_list,',');
            update_option('wc_settings_tab_dividebuy_tracking_carrier_list', $courier_list);
        }
        
        /**
         * This function creates duplicate order based on provided order id.
         */
        public function create_guarantor_order(){
            $post = trim(file_get_contents("php://input"));
            $postData = json_decode($post);

            $order_id = $postData->orderId;
            $order = new WC_Order( $order_id );            

            // Checking order product stock.
            $product_stock = $dividebuy_helper->check_order_product_stock($order);
            
            $order_is_dividebuy = get_post_meta($order->id, 'order_is_dividebuy');
            if($order_is_dividebuy[0] != "yes"){
                $result = array(
                    'error'     => 1,
                    'success'   => 0,
                    'order_id'  => $order->id,
                    'message'   => 'Non-DivideBuy order.',
                    'status'    => '402',
                );
            }else{
                if($product_stock == "true"){
                    $order->update_status('cancelled', 'Order has been canceled from DivideBuy.');
                    $order_id = $dividebuy_helper->create_duplicate_order($order);
                    wp_delete_post( $order->id, true );

                    if( $order_id != "" ){
                        $result = array(
                            'error'     => 0,
                            'success'   => 1,
                            'orderId'   => $order_id,
                            'status'    => '200'
                        );
                    }else{
                        $result = array(
                            'error'     => 1,
                            'success'   => 0,
                            'message'   => "There is some error in placing order.",
                            'status'    => '408'
                        );
                    }
                    
                }else{
                    $result = array(
                        'error'     => 1,
                        'success'   => 0,
                        'message'   => 'Order stock is not available.',
                        'status'    => '407'
                    );
                }
            }
            $this->send_response($result);
        }
        /**
        *This function is for create a cms page 
        *
        **/
        public function dividebuypage()
        {
            //handling the data
            $post = trim(file_get_contents("php://input"));
            $postData = json_decode($post);
            $flag = 0;
            if($postData->title !='')
            {
                //checking page is already created or not 
                $args = array(
                                'post_type'     =>  'page',
                                'post_status'   => 'publish',
                                'meta_query'    => array(
                                        'relation'  => 'AND',
                                        array(
                                                'key' => 'dividebuyPage',
                                                'value' =>  '1',
                                                'compare'   => '='
                                              )
                                      )
                            );
                $my_query = new WP_Query( $args );
                //echo $my_query->have_posts(); die();
                if( $my_query->have_posts() ) {
                  while( $my_query->have_posts() ) {
                    //post existing so updating the data instead of creating a page
                    $my_query->the_post();
                        $postid = get_the_ID();
                        $my_post = array();
                        $my_post['ID'] = $postid;
                        $my_post['post_content'] = $postData->content;
                        $my_post['post_title'] = $postData->title;
                        $my_post['post_status']   = 'publish';
                        wp_update_post( $my_post );
                        $flag = 1;
                  } // end while
                }else
                {
                    // Create page as it is not created or delted by admin
                    $my_post = array(
                    'post_title'    => wp_strip_all_tags( $postData->title),
                    'post_content'  => $postData->content,
                    'post_status'   => 'publish',
                    'post_type'     =>  'page'

                    );
                    // Insert the post into the database
                    $postid = wp_insert_post( $my_post );
                    if(isset($postid))
                    {
                        update_post_meta($postid,'dividebuyPage','1');
                        $flag = 1;
                    }
                }
                wp_reset_postdata();
            }
            //Response sending
            if($flag == 0)
            {
                //error response
                $result = array(
                    'error'     => 1,
                    'success'   => 0,
                    'message'   => "There is something wrong in update content.",
                    'status'    => '408'
                );
            }else
            {
                //sucess response
                $result = array(
                    'error'     => 0,
                    'success'   => 1,
                    'message'   => "ok",
                    'status'    => '200'
                );
            }
            //respons send function called
            $this->send_response($result);
        }
        /*
         * This function retrieves available shipping methods based on provided postcode.
         */
        public function verify_postcode(){
            $post = trim(file_get_contents("php://input"));
            $postData = json_decode($post);

            if(isset($postData)){
                $dividebuy_helper = new DivideBuy_Helper();
                $order_id = $postData->orderId;
                $user_postcode = $postData->userPostcode;

                // Removing all products from cart.
                $dividebuy_helper->remove_products_from_cart();

                $order = new WC_Order( $order_id );            
                $shipping_changed = $dividebuy_helper->get_order_items_shipping_packages($order);

                if($shipping_changed == "true"){
                    $result = array(
                        'error'     => 1,
                        'success'   => 0,
                        'message'   => "There is change in shipping methods.",
                        'status'    => '408'
                    );
                }else{
                    $result = array(
                        'error'     => 0,
                        'success'   => 1,
                        'message'   => "ok",
                        'status'    => '200'
                    );
                }

                // Remove cart items which has been added from order.
                WC()->cart->empty_cart();

                // Restoring cart items which has been removed previously.
                $dividebuy_helper->add_session_products_to_cart();

            }else{
                $result = array(
                    'error'     => 1,
                    'success'   => 0,
                    'message'   => "Please check entered data.",
                    'status'    => '408'
                );
            }

            $this->send_response($result);
        }


        /**
            *Curl Call function
        **/ 
        public function curl_call_api($url,$data,$method)
        {
            
            $headers = array('Content-type: application/json');
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_HTTPHEADER,$headers);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
            $response = curl_exec($ch);
            curl_close($ch);
            return $response;
        }
    /** 
         * Response Handler
     * This sends a JSON response to the browser
     */
    protected function send_response($response){
            header('content-type: application/json; charset=utf-8');
            echo json_encode($response);
    }
}
new DivideBuy_Api();